"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@hypernetlabs/utils");
const neverthrow_1 = require("neverthrow");
const rxjs_1 = require("rxjs");
class HypernetIFrameProxy extends utils_1.ParentProxy {
    constructor(element, iframeUrl, iframeName) {
        super(element, iframeUrl, iframeName);
        this.element = element;
        this.iframeUrl = iframeUrl;
        this.iframeName = iframeName;
        this.coreInitialized = false;
        this.isInControl = false;
        this._handshakePromise = null;
        this.onControlClaimed = new rxjs_1.Subject();
        this.onControlYielded = new rxjs_1.Subject();
        this.onPushPaymentSent = new rxjs_1.Subject();
        this.onPullPaymentSent = new rxjs_1.Subject();
        this.onPushPaymentReceived = new rxjs_1.Subject();
        this.onPullPaymentReceived = new rxjs_1.Subject();
        this.onPushPaymentUpdated = new rxjs_1.Subject();
        this.onPullPaymentUpdated = new rxjs_1.Subject();
        this.onPushPaymentDelayed = new rxjs_1.Subject();
        this.onPullPaymentDelayed = new rxjs_1.Subject();
        this.onBalancesChanged = new rxjs_1.Subject();
        this.onDeStorageAuthenticationStarted = new rxjs_1.Subject();
        this.onDeStorageAuthenticationSucceeded = new rxjs_1.Subject();
        this.onDeStorageAuthenticationFailed = new rxjs_1.Subject();
        this.onMerchantAuthorized = new rxjs_1.Subject();
        this.onMerchantDeauthorizationStarted = new rxjs_1.Subject();
        this.onAuthorizedMerchantUpdated = new rxjs_1.Subject();
        this.onAuthorizedMerchantActivationFailed = new rxjs_1.Subject();
        this.onMerchantIFrameDisplayRequested = new rxjs_1.Subject();
        this.onMerchantIFrameCloseRequested = new rxjs_1.Subject();
        this.onInitializationRequired = new rxjs_1.Subject();
        this.onPrivateCredentialsRequested = new rxjs_1.Subject();
        // Initialize the promise that we'll use to monitor the core
        // initialization status. The iframe will emit an event "initialized"
        // once the core is initialized, we'll use that to resolve this promise.
        this.waitInitializedPromise = new Promise((resolve) => {
            this._handshakePromise = this.handshake.then((child) => {
                // Subscribe to the message streams from the iframe,
                // and convert them back to RXJS Subjects.
                child.on("onControlClaimed", (data) => {
                    this.isInControl = true;
                    this.onControlClaimed.next(data);
                });
                child.on("onControlYielded", (data) => {
                    this.isInControl = false;
                    this.onControlYielded.next(data);
                });
                child.on("onPushPaymentSent", (data) => {
                    this.onPushPaymentSent.next(data);
                });
                child.on("onPullPaymentSent", (data) => {
                    this.onPullPaymentSent.next(data);
                });
                child.on("onPushPaymentReceived", (data) => {
                    this.onPushPaymentReceived.next(data);
                });
                child.on("onPullPaymentReceived", (data) => {
                    this.onPullPaymentReceived.next(data);
                });
                child.on("onPushPaymentUpdated", (data) => {
                    this.onPushPaymentUpdated.next(data);
                });
                child.on("onPullPaymentUpdated", (data) => {
                    this.onPullPaymentUpdated.next(data);
                });
                child.on("onPushPaymentDelayed", (data) => {
                    this.onPushPaymentDelayed.next(data);
                });
                child.on("onPullPaymentDelayed", (data) => {
                    this.onPullPaymentDelayed.next(data);
                });
                child.on("onBalancesChanged", (data) => {
                    this.onBalancesChanged.next(data);
                });
                child.on("onDeStorageAuthenticationStarted", () => {
                    this._displayCoreIFrame();
                    this.onDeStorageAuthenticationStarted.next();
                });
                child.on("onDeStorageAuthenticationSucceeded", () => {
                    this._closeCoreIFrame();
                    this.onDeStorageAuthenticationSucceeded.next();
                });
                child.on("onDeStorageAuthenticationFailed", () => {
                    this.onDeStorageAuthenticationFailed.next();
                });
                child.on("onMerchantAuthorized", (data) => {
                    this.onMerchantAuthorized.next(data);
                });
                child.on("onMerchantDeauthorizationStarted", (data) => {
                    this.onMerchantDeauthorizationStarted.next(data);
                });
                child.on("onAuthorizedMerchantUpdated", (data) => {
                    this.onAuthorizedMerchantUpdated.next(data);
                });
                child.on("onAuthorizedMerchantActivationFailed", (data) => {
                    this.onAuthorizedMerchantActivationFailed.next(data);
                });
                // Setup a listener for the "initialized" event.
                child.on("initialized", () => {
                    // Resolve waitInitialized
                    resolve();
                    // And mark us as initialized
                    this.coreInitialized = true;
                });
                child.on("onMerchantIFrameDisplayRequested", (data) => {
                    this._displayCoreIFrame();
                    this.onMerchantIFrameDisplayRequested.next(data);
                });
                child.on("onMerchantIFrameCloseRequested", (data) => {
                    this._closeCoreIFrame();
                    this.onMerchantIFrameCloseRequested.next(data);
                });
                child.on("onInitializationRequired", () => {
                    this.onInitializationRequired.next();
                });
                child.on("onPrivateCredentialsRequested", () => {
                    this.onPrivateCredentialsRequested.next();
                });
            });
        });
    }
    finalizePullPayment(_paymentId, _finalAmount) {
        throw new Error("Method not implemented.");
    }
    initialized() {
        // If the child is not initialized, there is no way the core can be.
        if (this.child == null) {
            return neverthrow_1.ok(false);
        }
        // Return the current known status of coreInitialized. We request this
        // information as soon as the child is up.
        return neverthrow_1.ok(this.coreInitialized);
    }
    waitInitialized() {
        return neverthrow_1.ResultAsync.fromSafePromise(this.waitInitializedPromise);
    }
    inControl() {
        // If the child is not initialized, there is no way the core can be.
        if (this.child == null) {
            return neverthrow_1.ok(false);
        }
        // Return the current known status of inControl.
        return neverthrow_1.ok(this.isInControl);
    }
    getEthereumAccounts() {
        return this._createCall("getEthereumAccounts", null);
    }
    initialize(account) {
        return this._createCall("initialize", account);
    }
    getPublicIdentifier() {
        return this._createCall("getPublicIdentifier", null);
    }
    depositFunds(assetAddress, amount) {
        return this._createCall("depositFunds", {
            assetAddress,
            amount: amount,
        });
    }
    withdrawFunds(assetAddress, amount, destinationAddress) {
        return this._createCall("withdrawFunds", {
            assetAddress,
            amount: amount,
            destinationAddress,
        });
    }
    getBalances() {
        return this._createCall("getBalances", null);
    }
    getLinks() {
        return this._createCall("getLinks", null);
    }
    getActiveLinks() {
        return this._createCall("getActiveLinks", null);
    }
    getLinkByCounterparty(_counterPartyAccount) {
        throw new Error("Unimplemented");
    }
    sendFunds(counterPartyAccount, amount, expirationDate, requiredStake, paymentToken, merchantUrl) {
        return this._createCall("sendFunds", {
            counterPartyAccount,
            amount: amount,
            expirationDate,
            requiredStake: requiredStake,
            paymentToken,
            merchantUrl,
        });
    }
    authorizeFunds(counterPartyAccount, totalAuthorized, expirationDate, deltaAmount, deltaTime, requiredStake, paymentToken, merchantUrl) {
        return this._createCall("authorizeFunds", {
            counterPartyAccount,
            totalAuthorized: totalAuthorized,
            expirationDate,
            deltaAmount: deltaAmount,
            deltaTime,
            requiredStake: requiredStake,
            paymentToken,
            merchantUrl,
        });
    }
    acceptOffers(paymentIds) {
        return this._createCall("acceptFunds", paymentIds);
    }
    pullFunds(paymentId, amount) {
        return this._createCall("pullFunds", {
            paymentId,
            amount: amount,
        });
    }
    initiateDispute(paymentId) {
        return this._createCall("initiateDispute", paymentId);
    }
    resolveInsurance(paymentId) {
        return this._createCall("resolveInsurance", paymentId);
    }
    mintTestToken(amount) {
        return this._createCall("mintTestToken", amount);
    }
    authorizeMerchant(merchantUrl) {
        return this._createCall("authorizeMerchant", merchantUrl);
    }
    deauthorizeMerchant(merchantUrl) {
        return this._createCall("deauthorizeMerchant", merchantUrl);
    }
    getAuthorizedMerchants() {
        return this._createCall("getAuthorizedMerchants", null);
    }
    getAuthorizedMerchantsConnectorsStatus() {
        return this._createCall("getAuthorizedMerchantsConnectorsStatus", null);
    }
    displayMerchantIFrame(merchantUrl) {
        return this.getAuthorizedMerchantsConnectorsStatus().andThen((merchantsMap) => {
            if (merchantsMap.get(merchantUrl) == true) {
                this._displayCoreIFrame();
                return this._createCall("displayMerchantIFrame", merchantUrl);
            }
            else {
                alert(`Merchant ${merchantUrl} is not activated at the moment, try again later`);
                return neverthrow_1.okAsync(undefined);
            }
        });
    }
    closeMerchantIFrame(merchantUrl) {
        this._closeCoreIFrame();
        return this._createCall("closeMerchantIFrame", merchantUrl);
    }
    providePrivateCredentials(privateKey, mnemonic) {
        return this._createCall("providePrivateCredentials", {
            privateKey,
            mnemonic,
        });
    }
    setPreferredPaymentToken(tokenAddress) {
        return this._createCall("setPreferredPaymentToken", tokenAddress);
    }
    getPreferredPaymentToken() {
        return this._createCall("getPreferredPaymentToken", null);
    }
    _displayCoreIFrame() {
        // Show core iframe
        if (this.child != null) {
            this.child.frame.style.display = "block";
        }
        // Show core iframe container
        if (this.element != null) {
            this.element.style.display = "block";
        }
    }
    _closeCoreIFrame() {
        // Hide core iframe
        if (this.child != null) {
            this.child.frame.style.display = "none";
        }
        // Hide core iframe container
        if (this.element != null) {
            this.element.style.display = "none";
        }
    }
}
exports.default = HypernetIFrameProxy;
//# sourceMappingURL=HypernetIFrameProxy.js.map